@@ -1,24 +1,12 @@
+from __future__ import division
 import numpy as np
-from ..tools.psd import psd_freq, cohere, psd, cpsd_quasisync
-from ..tools.misc import slice1d_along_axis
-from scipy.signal import detrend
-from .base import ma, rad_hz, TimeBased
-from h5py._hl.dataset import Dataset
-
-
-class TimeBindat(TimeBased):
-
-    """
-    A base, abstract class for binned data.
-
-    """
-    @property
-    def freq(self,):
-        return self.omega[:] / rad_hz
-
-    @freq.setter
-    def freq(self, val):
-        self.omega = val * rad_hz
+from ..tools.psd import psd_freq, cohere, psd, cpsd_quasisync, \
+    cpsd, phase_angle
+from ..tools.misc import slice1d_along_axis, detrend
+from .base import ma, TimeData
+import copy
+import warnings
+import six
 
 
 class TimeBinner(object):
@@ -51,7 +39,7 @@ class TimeBinner(object):
         Returns `outshape` (the 'reshape'd shape) for an `inshape` array.
         """
         n_bin = int(self._parse_nbin(n_bin))
-        return list(inshape[:-1]) + [inshape[-1] / n_bin, n_bin + n_pad]
+        return list(inshape[:-1]) + [int(inshape[-1] // n_bin), int(n_bin + n_pad)]
 
     def _outshape_fft(self, inshape, n_fft=None, n_bin=None):
         """
@@ -59,7 +47,7 @@ class TimeBinner(object):
         """
         n_fft = self._parse_nfft(n_fft)
         n_bin = self._parse_nbin(n_bin)
-        return list(inshape[:-1]) + [inshape[-1] / n_bin, n_fft / 2]
+        return list(inshape[:-1]) + [int(inshape[-1] // n_bin), int(n_fft // 2)]
 
     def _parse_fs(self, fs=None):
         if fs is not None:
@@ -105,27 +93,34 @@ class TimeBinner(object):
 
         """
         n_bin = self._parse_nbin(n_bin)
-        npd0 = n_pad / 2
-        npd1 = (n_pad + 1) / 2
+        npd0 = int(n_pad // 2)
+        npd1 = int((n_pad + 1) // 2)
         shp = self._outshape(arr.shape, n_pad=0, n_bin=n_bin)
         out = np.zeros(
             self._outshape(arr.shape, n_pad=n_pad, n_bin=n_bin),
             dtype=arr.dtype)
         if np.mod(n_bin, 1) == 0:
+            # n_bin needs to be int
+            n_bin = int(n_bin)
             # If n_bin is an integer, we can do this simply.
-            out[..., npd0: n_bin + npd0] = (arr[..., :(shp[-2] * shp[-1])]
-                                            ).reshape(shp, order='C')
+            out[..., npd0: n_bin + npd0] = (
+                arr[..., :(shp[-2] * shp[-1])]).reshape(shp, order='C')
         else:
-            inds = (np.arange(np.prod(shp[-2:])) * n_bin / int(n_bin)
+            inds = (np.arange(np.prod(shp[-2:])) * n_bin // int(n_bin)
                     ).astype(int)
-            out[..., npd0:int(n_bin) + npd0] = (arr[..., inds]
+            n_bin = int(n_bin)
+            out[..., npd0:n_bin + npd0] = (arr[..., inds]
                                                 ).reshape(shp, order='C')
-            # n_bin needs to be int for the n_pad operation.
             n_bin = int(n_bin)
         if n_pad != 0:
             out[..., 1:, :npd0] = out[..., :-1, n_bin:n_bin + npd0]
             out[..., :-1, -npd1:] = out[..., 1:, npd0:npd0 + npd1]
-        if ma.valid and out.__class__ is ma.marray:
+        if isinstance(arr, np.ma.MaskedArray):
+            out = np.ma.masked_where(self.reshape(arr.mask,
+                                                  n_pad=n_pad,
+                                                  n_bin=n_bin),
+                                     out)
+        if ma.valid and isinstance(out, ma.marray):
             out.meta.dim_names += ['time2']
         return out
 
@@ -135,9 +130,7 @@ class TimeBinner(object):
 
         ... Need to fix this to deal with NaNs...
         """
-        return detrend(self.reshape(dat, n_pad=n_pad, n_bin=n_bin),
-                       axis=-1,
-                       type='linear')
+        return detrend(self.reshape(dat, n_pad=n_pad, n_bin=n_bin), axis=-1)
 
     def demean(self, dat, n_pad=0, n_bin=None):
         """
@@ -146,12 +139,58 @@ class TimeBinner(object):
         dt = self.reshape(dat, n_pad=n_pad, n_bin=n_bin)
         return dt - (dt.mean(-1)[..., None])
 
-    def mean(self, dat, n_bin=None):
-        out = self.reshape(dat, n_bin=n_bin).mean(-1)
+    def mean(self, dat, axis=-1, n_bin=None, mask_thresh=None):
+        """Average an array object.
+
+        Parameters
+        ----------
+        n_bin : int (default is self.n_bin)
+
+        mask_thresh : float (between 0 and 1)
+            if the input data is a masked array, and mask_thresh is
+            not None mask the averaged values where the fraction of
+            bad points is greater than mask_thresh
+        """
+        # Can I turn this 'swapaxes' stuff into a decorator?
+        if axis != -1:
+            dat = np.swapaxes(dat, axis, -1)
+        n_bin = self._parse_nbin(n_bin)
+        tmp = self.reshape(dat, n_bin=n_bin)
+        out = tmp.mean(-1)
+        if isinstance(dat, np.ma.MaskedArray) and mask_thresh is not None:
+            out.mask = tmp.mask.sum(-1).astype(np.float) > mask_thresh * n_bin
+        if axis != -1:
+            np.swapaxes(out, axis, -1)
         if dat.__class__ is np.ndarray:
             return out
         return out.view(dat.__class__)
 
+    def mean_angle(self, dat, axis=-1, units='radians',
+                   n_bin=None, mask_thresh=None):
+        """Average an angle array.
+
+        Parameters
+        ----------
+        units : {'radians' | 'degrees'}
+
+        n_bin : int (default is self.n_bin)
+
+        mask_thresh : float (between 0 and 1)
+            if the input data is a masked array, and mask_thresh is
+            not None mask the averaged values where the fraction of
+            bad points is greater than mask_thresh
+        """
+        if units.lower().startswith('deg'):
+            dat = dat * np.pi / 180
+        elif units.lower().startswith('rad'):
+            pass
+        else:
+            raise ValueError("Units must be either 'rad' or 'deg'.")
+        return np.angle(self.mean(np.exp(1j * dat)))
+
+    def var(self, dat, n_bin=None):
+        return self.reshape(dat, n_bin=n_bin).var(-1)
+
     def std(self, dat, n_bin=None):
         return self.reshape(dat, n_bin=n_bin).std(-1)
 
@@ -169,15 +208,15 @@ class TimeBinner(object):
 
         """
         n_bin = self._parse_nbin(n_bin)
-        out = np.empty(self._outshape(indat.shape, n_bin=n_bin)[:-1]
-                       + [n_bin / 4], dtype=indat.dtype)
-        dt1 = self.reshape(indat, n_pad=n_bin/2 - 2)
+        out = np.empty(self._outshape(indat.shape, n_bin=n_bin)[:-1] +
+                       [int(n_bin // 4)], dtype=indat.dtype)
+        dt1 = self.reshape(indat, n_pad=n_bin / 2 - 2)
         # Here we de-mean only on the 'valid' range:
-        dt1 = dt1-dt1[...,:, n_bin / 4: -n_bin / 4].mean(-1)[..., None]
+        dt1 = dt1 - dt1[..., :, int(n_bin // 4):int(-n_bin // 4)].mean(-1)[..., None]
         dt2 = self.demean(indat)  # Don't pad the second variable.
         dt2 = dt2 - dt2.mean(-1)[..., None]
-        se = slice(int(n_bin / 4) - 1, None, 1)
-        sb = slice(int(n_bin / 4) - 1, None, -1)
+        se = slice(int(n_bin // 4) - 1, None, 1)
+        sb = slice(int(n_bin // 4) - 1, None, -1)
         for slc in slice1d_along_axis(dt1.shape, -1):
             tmp = np.correlate(dt1[slc], dt2[slc], 'valid')
             # The zero-padding in reshape means we compute coherence
@@ -195,9 +234,9 @@ class TimeBinner(object):
         if npt is None:
             npt = self.n_bin
         if one_sided:
-            return np.arange(npt / 2, dtype=np.float32)
+            return np.arange(int(npt // 2), dtype=np.float32)
         else:
-            return np.arange(npt, dtype=np.float32) - npt / 2
+            return np.arange(npt, dtype=np.float32) - int(npt // 2)
 
     def calc_xcov(self, indt1, indt2, npt=None,
                   n_bin1=None, n_bin2=None, normed=False):
@@ -213,7 +252,7 @@ class TimeBinner(object):
         tmp = int(n_bin2) - int(n_bin1) + npt
         dt1 = self.reshape(indt1, n_pad=tmp - 1, n_bin=n_bin1)
         # Note here I am demeaning only on the 'valid' range:
-        dt1 = dt1 - dt1[...,:, tmp/2:-tmp/2].mean(-1)[..., None]
+        dt1 = dt1 - dt1[..., :, int(tmp // 2):int(-tmp // 2)].mean(-1)[..., None]
         # Don't need to pad the second variable:
         dt2 = self.demean(indt2, n_bin=n_bin2)
         dt2 = dt2 - dt2.mean(-1)[..., None]
@@ -225,14 +264,13 @@ class TimeBinner(object):
                     n_bin2)[..., None]
         return out
 
-    def do_avg(self, rawdat, outdat, names=None):
+    def do_avg(self, rawdat, outdat=None, names=None, n_time=None):
         """
 
         Parameters
         ----------
         rawdat : raw_data_object
-           The raw data structure to be binned (if None, a basic
-           TimeBindat class is created).
+           The raw data structure to be binned
         outdat : avg_data_object
            The bin'd (output) data object to which averaged data is added.
         names : list of strings
@@ -240,22 +278,41 @@ class TimeBinner(object):
            all data in `rawdat` will be binned.
 
         """
-        n = len(rawdat)
-        for nm, dat, grp in rawdat.iter_wg():
-            mro = dat.__class__.__mro__
-            if ((names is None) or (nm in names)) and \
-               ((np.ndarray in mro) or (Dataset in mro)) and \
-               (dat.shape[-1] == n):
-                outdat.add_data(nm, self.mean(dat), grp)
+        props = {}
+        if n_time is None:
+            n_time = rawdat.n_time
+        if outdat is None:
+            outdat = type(rawdat)()
+            props['n_bin'] = self.n_bin
+            props['n_fft'] = self.n_fft
+        if names is None:
+            names = rawdat.keys()
+        for ky in names:
+            if isinstance(ky, six.string_types) and '.' in ky:
+                g, nm = ky.split('.', 1)
+                if g not in outdat:
+                    outdat[g] = type(rawdat[g])()
+                outdat[g][nm] = self.mean(rawdat[g][nm],
+                                          axis=rawdat[g]._time_dim)
+            elif isinstance(rawdat[ky], TimeData):
+                outdat[ky] = TimeData()
+                self.do_avg(rawdat[ky], outdat[ky], n_time=n_time)
+            elif (isinstance(rawdat[ky], np.ndarray) and
+                  rawdat[ky].shape[rawdat._time_dim] == n_time):
+                outdat[ky] = self.mean(rawdat[ky], axis=rawdat._time_dim)
+            else:
+                outdat[ky] = copy.deepcopy(rawdat[ky])
+        if 'props' in outdat:
+            outdat['props'].update(props)
+        return outdat
 
-    def do_var(self, rawdat, outdat, names=None, suffix='_var'):
+    def do_var(self, rawdat, outdat=None, names=None, suffix='_var'):
         """Calculate the variance of data attributes.
 
         Parameters
         ----------
         rawdat : raw_data_object
-           The raw data structure to be binned (if None, a basic
-           TimeBindat class is created).
+           The raw data structure to be binned.
 
         outdat : avg_data_object
            The bin'd (output) data object to which variance data is added.
@@ -265,13 +322,19 @@ class TimeBinner(object):
            `names` is None, all data in `rawdat` will be binned.
 
         """
-        n = len(rawdat)
-        for nm, dat, grp in rawdat.iter_wg():
-            mro = dat.__class__.__mro__
-            if ((names is None) or (nm in names)) and \
-               ((np.ndarray in mro) or (Dataset in mro)) and \
-               (dat.shape[-1] == n):
-                outdat.add_data(nm + suffix, self.reshape(dat).var(-1), grp)
+        if outdat is None:
+            outdat = TimeData()
+        if names is None:
+            names = rawdat.keys()
+        for ky in names:
+            if isinstance(rawdat[ky], TimeData):
+                outdat[ky] = TimeData()
+                self.do_avg(rawdat[ky], outdat[ky])
+            elif isinstance(rawdat[ky], np.ndarray):
+                outdat[ky] = self.reshape(rawdat[ky]).var(-1)
+            else:
+                outdat[ky] = copy.deepcopy(rawdat[ky])
+        return outdat
 
     def __init__(self, n_bin, fs, n_fft=None, n_fft_coh=None):
         """
@@ -299,19 +362,27 @@ class TimeBinner(object):
             print("n_fft larger than n_bin \
             doesn't make sense, setting n_fft=n_bin")
         if n_fft_coh is None:
-            self.n_fft_coh = self.n_bin / 6
+            self.n_fft_coh = int(self.n_bin // 6)
         elif n_fft_coh >= n_bin:
-            self.n_fft_coh = n_bin / 6
-            print("n_fft_coh >= n_bin doesn't make sense, \
-            setting n_fft_coh=n_bin/6")
+            self.n_fft_coh = int(n_bin // 6)
+            print("n_fft_coh must be smaller than n_bin, "
+                  "setting n_fft_coh=n_bin / 6")
 
-    def __call__(self, rawdat, out_type=TimeBindat):
-        outdat = out_type()
-        outdat.props['n_bin'] = self.n_bin
-        outdat.props['n_fft'] = self.n_fft
-        outdat.props['n_fft_coh'] = self.n_fft_coh
-        outdat.props.update(rawdat.props)
-        return outdat
+    def _check_indata(self, rawdat):
+        if np.any(np.array(rawdat.shape) == 0):
+            raise RuntimeError(
+                "The input data cannot be averaged "
+                "because it is empty.")
+        if 'DutyCycle_NBurst' in rawdat.props and \
+           rawdat.props['DutyCycle_NBurst'] < self.n_bin:
+            warnings.warn(
+                "The averaging interval (n_bin = {}) is "
+                "larger than the burst interval (NBurst = {})!"
+                .format(self.n_bin, rawdat.props['DutyCycle_NBurst']))
+        if rawdat['props']['fs'] != self.fs:
+            raise Exception(
+                "The input data sample rate (dat.fs) does not "
+                "match the sample rate of this binning-object!")
 
     def cohere(self, dat1, dat2, window='hann', debias=True,
                noise=(0, 0), n_fft=None, n_bin1=None, n_bin2=None,):
@@ -323,7 +394,7 @@ class TimeBinner(object):
         n_bin1 = self._parse_nbin(n_bin1)
         n_bin2 = self._parse_nbin(n_bin2)
         oshp = self._outshape_fft(dat1.shape, n_fft=n_fft, n_bin=n_bin1)
-        oshp[-2] = np.min([oshp[-2], dat2.shape[-1] / n_bin2])
+        oshp[-2] = np.min([oshp[-2], int(dat2.shape[-1] // n_bin2)])
         out = np.empty(oshp, dtype=dat1.dtype)
         # The data is detrended in psd, so we don't need to do it here.
         dat1 = self.reshape(dat1, n_pad=n_fft, n_bin=n_bin1)
@@ -359,15 +430,55 @@ class TimeBinner(object):
         n_bin1 = self._parse_nbin(n_bin1)
         n_bin2 = self._parse_nbin(n_bin2)
         oshp = self._outshape_fft(dat1.shape, n_fft=n_fft, n_bin=n_bin1)
-        oshp[-2] = np.min([oshp[-2], dat2.shape[-1] / n_bin2])
-        out = np.empty(oshp, dtype=dat1.dtype)
+        oshp[-2] = np.min([oshp[-2], int(dat2.shape[-1] // n_bin2)])
         # The data is detrended in psd, so we don't need to do it here:
         dat1 = self.reshape(dat1, n_pad=n_fft)
         dat2 = self.reshape(dat2, n_pad=n_fft)
+        out = np.empty(oshp, dtype='c{}'.format(dat1.dtype.itemsize * 2))
+        if dat1.shape == dat2.shape:
+            cross = cpsd
+        else:
+            cross = cpsd_quasisync
         for slc in slice1d_along_axis(out.shape, -1):
             # PSD's are computed in radian units:
-            out[slc] = cpsd_quasisync(dat1[slc], dat2[slc], n_fft,
-                                      2 * np.pi * fs, window=window)
+            out[slc] = cross(dat1[slc], dat2[slc], n_fft,
+                             2 * np.pi * fs, window=window)
+        return out
+
+    def phase_angle(self, dat1, dat2, window='hann',
+                    n_fft=None, n_bin1=None, n_bin2=None,):
+        """
+        Calculate the phase difference between two signals as a
+        function of frequency (complimentary to coherence).
+
+        Parameters
+        ----------
+        dat1    : np.ndarray
+          The first raw-data array of which to calculate the cpsd.
+        dat2    : np.ndarray
+          The second raw-data array of which to calculate the cpsd.
+        window : string
+          String indicating the window function to use (default: 'hanning').
+
+        Returns
+        -------
+        out : np.ndarray
+          The phase difference between signal dat1 and dat2.
+        """
+        if n_fft is None:
+            n_fft = self.n_fft_coh
+        n_bin1 = self._parse_nbin(n_bin1)
+        n_bin2 = self._parse_nbin(n_bin2)
+        oshp = self._outshape_fft(dat1.shape, n_fft=n_fft, n_bin=n_bin1)
+        oshp[-2] = np.min([oshp[-2], int(dat2.shape[-1] // n_bin2)])
+        # The data is detrended in psd, so we don't need to do it here:
+        dat1 = self.reshape(dat1, n_pad=n_fft)
+        dat2 = self.reshape(dat2, n_pad=n_fft)
+        out = np.empty(oshp, dtype='c{}'.format(dat1.dtype.itemsize * 2))
+        for slc in slice1d_along_axis(out.shape, -1):
+            # PSD's are computed in radian units:
+            out[slc] = phase_angle(dat1[slc], dat2[slc], n_fft,
+                                   window=window)
         return out
 
     def psd(self, dat, fs=None, window='hann', noise=0,
